/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.mapper.rdb.adapter.api;

import java.util.List;
import java.util.ArrayList;

/**
 * is a recursive structure representing join between tables.
 *  
 * @author S.Chassande-Barrioz
 */
public class JoinedTable {

	/**
	 * defines a join between two tables. It contains the list of columns names
	 * in table name
	 */
	public class Join {
		/**
		 * column names in the first table
		 */
		public List colT1;

		/**
		 * column names in the second table
		 */
		public List colT2;

		public Join() {
			colT1 = new ArrayList();
			colT2 = new ArrayList();
		}

		/**
		 * add a join condition (equality between columns)
		 * @param col1 is a column in the first table
		 * @param col2 is a column in the second table
		 */
		public void addJoinColumn(String col1, String col2) {
			colT1.add(col1);
			colT2.add(col2);
		}

		public Object clone() {
			Join j = new Join();
			colT1.addAll(colT1);
			colT2.addAll(colT2);
			return j;
		}
	}

	/**
	 * List of Join instaces. The Join instance at the N position describes
	 * the join between the current table (first table) and the JoinedTable
	 * at the position N in the list 'joinedTables' (second table).
	 * Therefore the joins list has the exactly the same size than the
	 * 'joinedTables' list.
	 * this is list can be empty if not joined table are linked to the current.
	 */
	public List joins;

	/**
	 * List of JoinedTable. The JoinedTable instance at the N position is linked
	 * to the current JoinedTable by the a join desribes by the Join instance
	 * at the position N in the list 'joins'.
	 * Therefore the joinedTables list has the exactly the same size than the
	 * 'joins' list.
	 * this is list can be empty if not joined table are linked to the current.
	 */
	public List joinedTables;

	/**
	 * the name of the current table. never null
	 */
	public String tableName;

	/**
	 * is the alias name the current table. It can be null if no alias is
	 * required.
	 */
	public String alias;

	/**
	 * no arg constructor required for cloning, but must not be used in usual
	 * case.
	 */
	public JoinedTable() {
	}

	/**
	 * Build a JoinedTable with a particular table name.
	 * @param tablename is the name of the table
	 */
	public JoinedTable(String tablename) {
		this(tablename, null);
	}


	/**
	 * Build a JoinedTable with a particular table name and alias name.
	 * @param tablename is the name of the table
	 * @param alias is the name of the alias
	 */
	public JoinedTable(String tablename, String alias) {
		tableName = tablename;
		this.alias = alias;
		joins = new ArrayList();
		joinedTables = new ArrayList();
	}

	/**
	 * Creates a new Join instance to a new JoinedTable
	 * @param tablename is the name of the new JoinedTable
	 * @return the Join instance representing the join between the current table
	 * and the created JoinedTable.
	 */
	public Join createChildren(String tablename) {
		return createChildren(tablename, null);
	}

	/**
	 * Creates a new Join instance to a new JoinedTable
	 * @param tablename is the name of the new JoinedTable
	 * @param alias is the alias name of the new JoinedTable
	 * @return the Join instance representing the join between the current table
	 * and the created JoinedTable.
	 */
	public Join createChildren(String tablename, String alias) {
		return createChildren(new JoinedTable(tablename, alias));
	}

	/**
	 * Creates a new Join instance to a given JoinedTable
	 * @param jt is a JoinedTable
	 * @return the Join instance representing the join between the current table
	 * and the given JoinedTable.
	 */
	public Join createChildren(JoinedTable jt) {
		joinedTables.add(jt);
		Join j = new Join();
		joins.add(j);
		return j;
	}

	/**
	 * @return a boolean value indicating if a given JoinedTable is the
	 * joinedtables of the current JoinedTable
	 */
	public boolean contains(JoinedTable jt) {
		if (joinedTables.indexOf(jt) != -1) {
			return true;
		}
		for (int i = 0; i < jt.joinedTables.size(); i++) {
			if (((JoinedTable) jt.joinedTables.get(i)).contains(jt)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Clone the current JoinedTable with a context.
	 * @param clone is the clone instance to fill with the value of the current
	 * instance. If this parameter is null, the implementation has o create a
	 * new one.
	 * @param obj2clone is a map between object already cloned and their clone
	 * instance
	 * @return the clone instance of the current instance.
	 * @throws java.lang.CloneNotSupportedException
	 */
	public Object clone(Object clone,
						java.util.Map obj2clone) throws CloneNotSupportedException {
		if (clone == null) {
			try {
				clone = getClass().newInstance();
			} catch (Exception e) {
				e.printStackTrace();
				throw new CloneNotSupportedException("Impossible to clone a "
					+ getClass().getName() + " instance: " + e.getMessage());
			}
		}
		if (obj2clone != null) {
			obj2clone.put(this, clone);
		}
		JoinedTable jt = (JoinedTable) clone;
		jt.tableName = tableName;
		jt.alias = alias;
		if (jt.joinedTables != null) {
			jt.joinedTables = new ArrayList();
			jt.joins = new ArrayList();
			for(int i=0; i<joinedTables.size(); i++) {
				jt.joins.add(((Join) joins.get(i)).clone());
				jt.joinedTables.add(
					getClone((JoinedTable) joinedTables.get(i), obj2clone));
			}
		}
		return clone;
	}

	public static Object getClone(JoinedTable obj,
						   java.util.Map obj2clone) throws CloneNotSupportedException {
		if (obj == null) {
			return null;
		}
		Object o = obj2clone.get(obj);
		if (o == null) {
			o = obj.clone(null, obj2clone);
		}
		return o;
	}
}
